@@ -20,6 +20,7 @@ _delay_import_(globals(),"..utils","util","cutils","nested_array","context")
 _delay_import_(globals(),"..itypes","detector","type_attribute_freeze","convertors","dimensions")
 _delay_import_(globals(),"..repops_slice")
 _delay_import_(globals(),"..repops_dim")
+_delay_import_(globals(),"..repops_funcs")
 _delay_import_(globals(),"..utils.missing","Missing")
 
 
@@ -27,7 +28,7 @@ class PyRepresentor(wrapper.SourceRepresentor):
     def __init__(self, slices):
         self._initialize(slices)
 
-def Rep(data=None, dtype=None, unpack=True, name=None):
+def Rep(data=None, dtype=None, unpack=True, name=None, allow_convert=False):
     """Packs python data structures into a :py:class:`ibidas.representor.Representor` object.
         
        :param data: any python object
@@ -99,9 +100,11 @@ def Rep(data=None, dtype=None, unpack=True, name=None):
     res = wrapper.SourceRepresentor()
     res._initialize((data_slice,)) 
 
-    if(unpack and isinstance(res.getType(), rtypes.TypeTuple)):
+    if(unpack and isinstance(res._getType(), rtypes.TypeTuple)):
         res = repops_slice.UnpackTuple(res)
-    
+   
+    if allow_convert:
+        res = res.Detect()
     return res
 
 
@@ -148,7 +151,7 @@ class ResultOp(ops.DataOp):
         res += '\n' + r
         return res
 
-class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE), 
+class PyExec(VisitorFactory(prefixes=("visit","unpackCast"), flags=NF_ELSE), 
              DirectVisitorFactory(prefixes=("cast",),flags=NF_ERROR), manager.Pass):
 
     after = set([create_graph.CreateGraph, serialize_exec.SerializeExec])
@@ -240,7 +243,50 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
             return slice.modify(rtype=det.getType())
         else:
             return slice
-   
+    
+    def visitDetectAndCastOp(self,node,slice):
+        det = detector.Detector(allow_need_convert=node.allow_convert)
+        det.setParentDimensions(node.dims)
+        det.processSeq(slice.data.flat())
+        nslice = slice.modify(rtype=det.getType())
+        return self.unpackCast(nslice.type, slice, nslice)
+
+    def unpackCastTypeTuple(self, rtype, oslice, nslice):
+        nslices = []
+        for i in range(len(rtype.fieldnames)):
+            tnnode = ops.UnpackTupleOp(nslice, i)
+            toslice = self.visitUnpackTupleOp(tnnode, oslice)
+            toslice = self.unpackCast(tnnode.type, toslice, tnnode)
+            nslices.append(toslice)
+        newnode = ops.PackTupleOp(tuple(nslices), nslice.name)
+        return self.visitPackTupleOp(newnode, nslices)
+
+    def unpackCastTypeArray(self, rtype, oslice, nslice):
+        nnode = ops.UnpackArrayOp(nslice)
+
+        toslice = self.visitUnpackArrayOp(nnode, oslice)
+        toslice = self.unpackCast(nnode.type, toslice, nnode)
+        
+        onode = ops.PackArrayOp(toslice)
+        return self.visitPackArrayOp(onode, toslice)
+    
+    def unpackCastTypeSet(self, rtype, oslice, nslice):
+        nnode = ops.UnpackArrayOp(nslice)
+
+        toslice = self.visitUnpackArrayOp(nnode, oslice)
+        toslice = self.unpackCast(nnode.type, toslice, nnode)
+        
+        onode = ops.UnaryFuncAggregateOp('Set',repops_funcs.setsig, repops_funcs.Param(nslice.name, nslice.type), 1, toslice)
+        return self.visitUnaryFuncAggregateOp(onode, toslice)
+    
+    
+    def unpackCastTypeScalar(self, rtype, oslice, nslice):
+        newnode = ops.CastOp(oslice, nslice.type)
+        return self.visitCastOp(newnode, oslice)
+    
+    unpackCastTypeString = unpackCastTypeScalar
+    unpackCastTypeAny = unpackCastTypeScalar
+
     def visitUnpackArrayOp(self,node,slice):
         ndata=slice.data.unpack(node.unpack_dims, subtype=node.type)
         return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
@@ -532,7 +578,15 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         if not node.type.has_missing:
             return slice.data.mapseq(lambda x:numpy.cast[dtype](x),res_type=node.type)
         else:
-            raise RuntimeError, "Casting missing values not yet supported"
+            xtype = dtype.type
+            def val(v):
+                try:
+                    return xtype(v)
+                except ValueError:
+                    return Missing
+
+            return slice.data.map(val, res_type=node.type)
+            #raise RuntimeError, "Casting missing values not yet supported"
 
     def caststring_to_int(self, castname, node, slice):
         if(node.type.has_missing):
@@ -568,6 +622,14 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
 
         return slice.data.mapseq(lambda x: func(x,dtype),res_type=node.type)
        
+    def caststring_to_string(self, castname, node, slice):
+        if(node.type.has_missing):
+            func = string_to_string_missing
+        else:
+            func = string_to_string
+        dtype = node.type.getPythonType()
+
+        return slice.data.mapseq(lambda x: func(x,dtype),res_type=node.type)
 
     def withinWithin(self, data, type1, type2, typeo, op, bcdepth=1):
         data1,data2 = data
@@ -1198,6 +1260,21 @@ def string_to_real_missing(seq, dtype):
 def string_to_real(seq, dtype):
     return util.darray([float(elem) for elem in seq],dtype)
 
+def string_to_string_missing(seq, dtype):
+    res = []
+    for elem in seq:
+        if elem is Missing or elem == "":
+            res.append(Missing)
+        else:
+            try:
+                res.append(dtype(elem))
+            except ValueError:
+                res.append(Missing)
+    return util.darray(res,object)
+
+def string_to_string(seq, dtype):
+    return util.darray([dtype(elem) for elem in seq],object)
+
 
 def none_to_missing(seq,stype):
     missing = stype.toMissingval()
